////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright:      2001/03/01; Andreas Huber <andreas@huber.net>; Zurich; Switzerland
////////////////////////////////////////////////////////////////////////////////////////////////////
// License:        This file is part of the downloadable accompanying material for the article
//                 "Elegant Function Call Wrappers" in the May 2001 issue of the C/C++ Users Journal
//                 (www.cuj.com). The material may be used, copied, distributed, modified and sold
//                 free of charge, provided that this entire notice (copyright, license and
//                 feedback) appears unaltered in all copies of the material.
//                 All material is provided "as is", without express or implied warranty.
////////////////////////////////////////////////////////////////////////////////////////////////////
// Feedback:       Please send comments to andreas@huber.net.
////////////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////////////////////////
// Below is the code discussed in the article
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <iostream>



////////////////////////////////////////////////////////////////////////////////////////////////////
class Functor
{
  public:
    virtual ~Functor() {}
    virtual void operator()() = 0;
};


////////////////////////////////////////////////////////////////////////////////////////////////////
template< class CalleePtr, class MemFunPtr, class Parm1 >
class MemberFunctor1 : public Functor
{
  public:
    ////////////////////////////////////////////////////////////////////////////////////////////////
    MemberFunctor1(
      const CalleePtr & pCallee,
      const MemFunPtr & pFunction,
      Parm1 aParm1
    ) :
      pCallee( pCallee ),
      pFunction( pFunction ),
      aParm1( aParm1 )
    {
    }

    virtual void operator()()
    {
      if ( ( pCallee != NULL ) && ( pFunction != NULL ) )
      {
        ( ( *pCallee ).*pFunction )( aParm1 );
      }
    }

  private:
    ////////////////////////////////////////////////////////////////////////////////////////////////
    CalleePtr pCallee;
    MemFunPtr pFunction;
    Parm1 aParm1;
};


////////////////////////////////////////////////////////////////////////////////////////////////////
template< class CalleePtr, class Callee, class Ret, class Type1, class Parm1 >
inline Functor * DeferCall(
  const CalleePtr & pCallee,
  Ret ( Callee::*pFunction )( Type1 ),
  const Parm1 & rParm1
)
{
  return new MemberFunctor1< CalleePtr, Ret ( Callee::* )( Type1 ), Parm1 >(
    pCallee, pFunction, rParm1 );
}


////////////////////////////////////////////////////////////////////////////////////////////////////
template< class Type >
class Referencer
{
  public:
    ////////////////////////////////////////////////////////////////////////////////////////////////
    Referencer( Type & rType ) : rType( rType ) {}
    operator Type &() const { return rType; }

  private:
    ////////////////////////////////////////////////////////////////////////////////////////////////
    Type & rType;
};

////////////////////////////////////////////////////////////////////////////////////////////////////
template< class Type >
inline Referencer< Type > Reference(
  Type & rType
)
{
  return Referencer< Type >( rType );
}


////////////////////////////////////////////////////////////////////////////////////////////////////
class Foo
{
  public:
    void Bar( const int & rVal ) { std::cout << rVal; }
};


////////////////////////////////////////////////////////////////////////////////////////////////////
int main( int argc, char* argv[] )
{
  Foo aFoo;
  float aValue = 1.0;


  ////////////////////////////////////////////////////////////////////////////////////////////////////
  // The following options are discussed in the article.
  // Exactly one of the options has to be "uncommented" for the test program to work.
  ////////////////////////////////////////////////////////////////////////////////////////////////////
  // option 3
  // really ugly!
  typedef MemberFunctor1< Foo *, void ( Foo::* )( const int & ), float > FooBarFunctor;
  Functor * pCall = new FooBarFunctor( &aFoo, &Foo::Bar, aValue );

  // option 4a
  // Functor * pCall = DeferCall( &aFoo, &Foo::Bar, aValue );

  // option 4b
  // Functor * pCall = DeferCall( &aFoo, &Foo::Bar, Reference( aValue ) );


  ++aValue;

  // the following would be done by the worker thread
  // pop call out of queue
  ( *pCall )();
  return 0;
}

