/* ****************************************************************************
 *
 *  MakefileGen.cpp
 *  Created by John F. Hubbard, on Fri Aug 11 2000, 14:30:27 PST
 *
 *  Copyright (c) 2000, ATD Azad Technology Development Corporation
 *
 *            The Reliable Software Outsource Resource
 *        You hire us, we do it for you, and we do it right.
 *
 *                       www.azadtech.com
 *
 *  Permission is granted to use this code without restriction,
 *  as long as this copyright notice appears in all source files.
 *
 *  File Contents: Implementation of the MakefileGen class.
 *  Please see MakefileGen.h for documentation of this class.
 *
 * %version: 2.30 %
 * %date_modified: Sat Nov 18 21:00:02 2000 %
 * %created_by: jhubbard %
 *
 *************************************************************************** */
#ifdef WIN32
#pragma warning (disable : 4786) // disable MSVC's useless template length warning
#endif // WIN32

#include <string>
#include <iostream>
#include "MakefileGen.h"

namespace code_gen
{

using std::endl;

MakefileGen::MakefileGen(const DigestedCommands& commands)
						 throw(std::string)
	: 	CodeGenerator	(commands),
		mWriter			(commands.GetMakefile(),
					 	 commands.GetWriteSentryMode(),
						 commands.WillCreateMakefile())
{
}


MakefileGen::~MakefileGen()
{

}

std::ostream&
MakefileGen::GetOstream()
{
	return mWriter.GetOstream();
}

void
MakefileGen::GenCommentHeader() 	throw(std::string)
{
	GetOstream() <<
	"#########################################################################" 	<< endl <<
    "#  " << mCommands.GetMakefile() << ""							 			   	<< endl <<
    "#  Created by " << mCommands.GetAuthor() << " on " << DateStamp()				<< endl <<
	"#" 	  																		<< endl <<
	"#  File Contents: makefile for the " << mCommands.GetBaseFilename() << " component." << endl <<
	"#" 																			<< endl <<
	"#########################################################################" 	<< endl <<
    " "																				<< endl;
}

void
MakefileGen::GenPreface() 			throw(std::string)
{
	GetOstream() <<
    "NAME=" << mCommands.GetBaseFilename()	 << endl;

	GetOstream() <<
    "INCLUDES=-I ${SXL_ROOT}/include"												    << endl <<
    "SOLARIS_LIBS=${SXL_ROOT}/lib/sxl.a"											    << endl <<
    "CXX=gcc"																		    << endl <<
    "LINK=gcc"																		    << endl <<
    "SOLARIS_LINK_FLAGS=-lgcc"														    << endl <<
    " "																					<< endl <<
    "all: ut${NAME} html"																<< endl <<
    " "																					<< endl <<
    "html: ${NAME}.h "																	<< endl <<
    "\tdoc++ -d html *.h" 																<< endl <<
    "\ttouch html"																		<< endl <<
    " "																					<< endl <<
    "# Solaris builds:"																	<< endl <<
    "LINK_FLAGS=  ${SOLARIS_LINK_FLAGS}"												<< endl <<
    "LIBS=        ${SOLARIS_LIBS}"														<< endl <<
    "CXX_FLAGS=-DSOLARIS_SPARC -g -c -fhonor-std \\"   									<< endl <<
    "          -fno-builtin -D_REENTRANT -Wall -Wno-unknown-pragmas -Wno-unused"		<< endl <<
    " "																					<< endl <<
    "ut${NAME} : ut${NAME}.o ${NAME}.o"													<< endl <<
    "\t${LINK} ${LINK_FLAGS} -o ut${NAME} ut${NAME}.o ${NAME}.o ${LIBS}"				<< endl <<
    "${NAME}.o : ${NAME}.cpp ${NAME}.h";

	if (mCommands.IccFile())
	{	GetOstream() << " ${NAME}.icc";
	}
	GetOstream() << endl <<
    "\t${CXX} ${CXX_FLAGS} ${INCLUDES} -c -o ${NAME}.o ${NAME}.cpp"						<< endl <<
    " "																					<< endl <<
    "ut${NAME}.o : ut${NAME}.cpp ${NAME}.h";

	if (mCommands.IccFile())
	{	GetOstream() << " ${NAME}.icc";
	}
	GetOstream() << endl <<
    "\t${CXX} ${CXX_FLAGS} ${INCLUDES} -c -o ut${NAME}.o ut${NAME}.cpp"					<< endl <<
    " "																					<< endl <<
    "clean:"																			<< endl <<
    "\trm *.o ut${NAME} html/" "*"	  													<< endl <<
    "\trmdir html"																		<< endl;
}

} //end of code_gen namespace
